/*************************************************************************
*  ISR.c
*  This module handel all ISR functions
**************************************************************************/
#include "ISR.h"
#include "McControlVariables.h"
#include "SVM.h"
//------------------------------------------------------------------------
signed int MotorSpeed;
float  float_measuredspeed;
//-------------------------------------------------------------------------
// CAN module RX/Tx Interrupt  ISR
//-------------------------------------------------------------------------
void C1Interrupt(void)iv IVT_ADDR_C1INTERRUPT
{
  IFS2bits.C1IF = 0;  // clear ECAN interrupt flag
  if(C1INTFbits.TBIF){// was it tx interrupt?
     C1INTFbits.TBIF = 0;}// if yes clear tx interrupt flag

  if(C1INTFbits.RBIF){ // was it rx interrupt?
     C1INTFbits.RBIF = 0;} // if yes clear rx interrupt flag
}
//------------------------------------------------------------------------
//Timer1 interrupt
//------------------------------------------------------------------------
void Timer1Int(void) iv IVT_ADDR_T1INTERRUPT
{
   IFS0bits.T1IF = 0;
   Period = ActualCapture - PastCapture;  // This is an UNsigned substraction
                                          // to get the Period between one
                                          // hall effect sensor transition

 // These operations limit the Period value to a range from 60 to 6000 rpm
    if (Period < (unsigned int)MINPERIOD)  // MINPERIOD or 6000 rpm
         Period = MINPERIOD;
    else if (Period > (unsigned int)MAXPERIOD) // MAXPERIOD or 60 rpm
             Period = MAXPERIOD;
 //---------------------------------------------------------------------
 // PhaseInc is a value added to the Phase variable to generate the sine
 // voltages. 1 electrical degree corresponds to a PhaseInc value of 184,
 // since the pointer to the sine table is a 16bit value, where 360 Elec
 // Degrees represents 65535 in the pointer.
 // __builtin_divud(Long Value, Int Value) is a function of the compiler
 // to do Long over Integer divisions.
 //PhaseInc = __builtin_divud(512000UL, Period);// Phase increment is used
 // by the PWM isr (SVM)
    PhaseInc = (signed int)(512000UL/Period);
 //------------------------------------------------------------------------
 // This subroutine in assembly calculates the MeasuredSpeed using
 // fractional division.
    //                   MINPERIOD (in fractional)
    //  MeasuredSpeed = ---------------------------
    //                    Period (in fractional)
 //-------------------------------------------------------------------
     float_measuredspeed = (float)MINPERIOD/Period ;
     MeasuredSpeed = Q15_Ftoi(float_measuredspeed);
 //---------------------------------------------------------------------
 // MeasuredSpeed sign adjustment based on current motor direction of
 // rotation
 //-----------------------------------------------------------------------
   if (Current_Direction == CCW)
      { MeasuredSpeed = -MeasuredSpeed ;}
 //----------------------------------------------------------------------
 // The following values represent the MeasuredSpeed values from the
 // previous operations:
 //
 // CONDITION        RPM          SFRAC16      SINT      HEX
 // Max Speed CW  -> 6000 RPM  -> 0.996805  -> 32663  -> 0x7F97
 // Min Speed CW  -> 60 RPM    -> 0.009984  -> 327    -> 0x0147
 // Min Speed CCW -> -60 RPM   -> -0.009984 -> -327   -> 0xFEB9
 // Max Speed CCW -> -6000 RPM -> -0.996805 -> -32663 -> 0x8069
 //------------------------------------------------------------------------
  SpeedControl(); // Speed PID controller is called here. It will use
                  // MeasuredSpeed, RefSpeed, some buffers and will generate
                  // the new ControlOutput, which represents a new amplitude
                  // of the sinewave that will be generated by the SVM
                 // subroutine.
#ifdef PHASE_ADVANCE
 // Calculate Phase Advance Based on Actual Speed and MAX_PH_ADV define
 // The following assembly instruction perform the following formula
 // using fractional multiplication:
 //
 // PhaseAdvance = MAX_PH_ADV * MeasuredSpeed
 //
   A = DSP_MPY(_MAX_PH_ADV,MeasuredSpeed,0, 0, 0, 0,0, 0);
   PhaseAdvance  = DSP_SAC(A,0);
#endif

   MotorStalledCounter++;// We increment a timeout variable to see if the
                         // motor is too slow (not generating hall effect
                         // sensors interrupts frequently enough) or if
                         // the motor is stalled. This variable is cleared
                         // in halls ISRs
  if ((MotorStalledCounter % _10MILLISEC) == 0)
   {
      ForceCommutation();// Force Commutation if no hall sensor changes
                           // have occured in specified timeout.
   }
  else if (MotorStalledCounter >= _100MILLISEC)
   {
     StopMotor(); // Stop motor is no hall changes have occured in
                  // specified timeout
   }
}
//------------------------------------------------------------------------
// Hall sensor A Interrupt
//-----------------------------------------------------
void IC1Interrupt(void) iv IVT_ADDR_IC1INTERRUPT
{
  IFS0bits.IC1IF = 0;        // Clear interrupt flag
  HallValue = (unsigned int)((PORTB >> 1) & 0x0007); // Read halls
  Sector = SectorTable[HallValue];        // Get Sector from table

 // This MUST be done for getting around the HW slow rate
  if (Sector != LastSector)
  {
    // Since a new sector is detected, clear variable that would stop
    // the motor if stalled.
     MotorStalledCounter = 0;
   // Motor current direction is computed based on Sector
   if ((Sector == 5) || (Sector == 2))
       Current_Direction = CCW;
    else
      Current_Direction = CW;
   // Motor commutation is actually based on the required direction, not
   // the current dir. This allows driving the motor in four quadrants
    if (Required_Direction == CW)
     {
          Phase = PhaseValues[Sector];
     }
     else
      {
       // For CCW an offset must be added to compensate difference in
       // symmetry of the sine table used for CW and CCW
       Phase = PhaseValues[(Sector + 3) % 6] + PhaseOffset;
      }
      LastSector = Sector; // Update last sector
   }
}
//------------------------------------------------------------------------
// Hall sensor B intrrupt
//-----------------------------------------------------
void IC2Interrupt(void) iv IVT_ADDR_IC2INTERRUPT
{
  IFS0.IC2IF = 0;        // Clear interrupt flag
  HallValue = (unsigned int)((PORTB >> 1) & 0x0007);        // Read halls
  Sector = SectorTable[HallValue];    // Get Sector from table
 // This MUST be done for getting around the HW slow rate
  if (Sector != LastSector)
  {
   // Calculate Hall period corresponding to half an electrical cycle
   PastCapture = ActualCapture;
   ActualCapture = IC2BUF;
   // Since a new sector is detected, clear variable that would stop
   // the motor if stalled.
   MotorStalledCounter = 0;
   // Motor current direction is computed based on Sector
   if ((Sector == 3) || (Sector == 0))
     Current_Direction = CCW;
   else
    Current_Direction = CW;
    // Motor commutation is actually based on the required direction, not
   // the current dir. This allows driving the motor in four quadrants
   if (Required_Direction == CW)
    {
      Phase = PhaseValues[Sector];
    }
    else
    {
     // For CCW an offset must be added to compensate difference in
     // symmetry of the sine table used for CW and CCW
     Phase = PhaseValues[(Sector + 3) % 6] + PhaseOffset;
    }
      LastSector = Sector; // Update last sector
   }
}
//------------------------------------------------------------------------
// Hall sensor C interrupt
//------------------------------------------------------
void IC7Interrupt(void) iv IVT_ADDR_IC7INTERRUPT
{
  IFS1.IC7IF = 0;        // Clear interrupt flag
  HallValue = (unsigned int)((PORTB >> 1) & 0x0007);// Read halls
  Sector = SectorTable[HallValue];        // Get Sector from table
  // This MUST be done for getting around the HW slow rate
  if (Sector != LastSector)
   {
    // Since a new sector is detected, clear variable that would stop
    // the motor if stalled.
    MotorStalledCounter = 0;
    // Motor current direction is computed based on Sector
    if ((Sector == 1) || (Sector == 4))
       Current_Direction = CCW;
     else
       Current_Direction = CW;
    // Motor commutation is actually based on the required direction, not
    // the current dir. This allows driving the motor in four quadrants
    if (Required_Direction == CW)
    {
     Phase = PhaseValues[Sector];
    }
    else
    {
      // For CCW an offset must be added to compensate difference in
      // symmetry of the sine table used for CW and CCW
      Phase = PhaseValues[(Sector + 3) % 6] + PhaseOffset;
    }
     LastSector = Sector; // Update last sector
   }
}
//-----------------------------------------------------------------------
// PWM Interrupt
//-----------------------------------------------------------------------
void PmwInterrupt(void) iv IVT_ADDR_MPWM1INTERRUPT
{
  IFS3bits.PWM1IF = 0;// Clear interrupt flag
 if (Required_Direction == CW)
  {
    if (Current_Direction == CW)
       Phase += PhaseInc;   // Increment Phase if CW to generate the
                             // sinewave only if both directions are equal
// If Required_Direction is CW (forward) POSITIVE voltage is applied
  #ifdef PHASE_ADVANCE
     asm nop;
     SVM(ControlOutput, Phase + PhaseAdvance);// PhaseAdvance addition
                                             // produces the sinewave
                                             // phase shift
   #else
       asm nop;
        SVM(ControlOutput,Phase);
     #endif
  }
  else
   {
    if (Current_Direction == CCW)
        Phase -= PhaseInc; // Decrement Phase if CCW to generate
                           // the sinewave only if both
                           // directions are equal
      // If Required_Direction is CCW (reverse) NEGATIVE voltage is applied
    #ifdef PHASE_ADVANCE
      SVM(-(ControlOutput+1),Phase + PhaseAdvance);//PhaseAdvance addition
                                                   //produces the sinewave
                                                   // phase shift
    #else
       SVM(-(ControlOutput+1), Phase);
    #endif
   }
}
//-------------------------------------------------------------------------
//ADC Interrupt
//-------------------------------------------------------------------------
void AdcInterrupt(void) iv IVT_ADDR_ADC1INTERRUPT
{
   IFS0bits.AD1IF = 0;      // Clear interrupt flag
   RefSpeed = ADC1BUF0; // Read POT value to set Reference Speed

}
//-------------------------------------------------------------------------
void SpeedControl(void)
{
   //----------------------------------------------------------
  // For Close loop Speed control implement PID Controller here
  //------------------------------------------------------------
  
  
 // If CLOSED_LOOP is undefined (running open loop) overide ControlOutput
 // with value read from the external potentiometer
 #ifndef CLOSED_LOOP
    ControlOutput = RefSpeed;
 #endif
// ControlOutput will determine the motor required direction
  if (ControlOutput < 0)
      Required_Direction = CCW;
  else
      Required_Direction = CW;

}
//-------------------------------------------------------------------------
void ForceCommutation(void)
{
  HallValue = (unsigned int)((PORTB >> 1) & 0x0007);  // Read halls
  Sector = SectorTable[HallValue];        // Read sector based on halls
  if (Sector != -1)        // If the sector is invalid don't do anything
   {
     // Depending on the required direction, a new phase is fetched
    if (Required_Direction == CW)
    {
          // Motor is required to run forward, so read directly the table
          Phase = PhaseValues[Sector];
    }
    else
     {
      // Motor is required to run reverse, so calculate new phase and
      // add offset to compensate asymmetries
      Phase = PhaseValues[(Sector + 3) % 6] + PhaseOffset;
     }
   }
}
//-------------------------------------------------------------------------
// Run Motor
//-------------------------------------------------------------------------
void RunMotor(void)
{
  ChargeBootstraps();
  TMR1 = 0;                        // Reset timer 1 for speed control
  TMR3 = 0;                        // Reset timer 3 for speed measurement
  ActualCapture = MAXPERIOD;      // Initialize captures for minimum speed
                                //(60 RPMs)
  PastCapture = 0;
  // Initialize direction with required direction
  // Remember that ADC is not stopped.
  HallValue = (unsigned int)((PORTB >> 1) & 0x0007);// Read halls
  LastSector = Sector = SectorTable[HallValue];    // Initialize Sector
                                                // variable
// RefSpeed's sign will determine if the motor should be run at CW
// (+RefSpeed) or CCW (-RefSpeed) ONLY at start up, since when the motor
// has started, the required direction will be set by the control output
// variable to be able to operate in the four quadrants
  if (RefSpeed < 0)
   {
     ControlOutput = 0;        // Initial output voltage
     Current_Direction = Required_Direction = CCW;
     Phase = PhaseValues[(Sector + 3) % 6] + PhaseOffset;
   }
  else
   {
     ControlOutput = 0;        // Initial output voltage
     Current_Direction = Required_Direction = CW;
     Phase = PhaseValues[Sector];
   }

  MotorStalledCounter = 0;  // Reset motor stalled protection counter
 // Set initial Phase increment with minimum value. This will change if a
 // costing operation is required by the application
   PhaseInc = (signed int)(512000UL/MAXPERIOD);

// Clear all interrupts flags
   IFS0.T1IF = 0;    // Clear timer 1 flag
   IFS0bits.IC1IF = 0;     // Clear interrupt flag  IC1,IC2;IC7
   IFS0bits.IC2IF = 0;
   IFS1bits.IC7IF = 0;
   IFS3bits.PWM1IF = 0;  // Clear interrupt flag
   // enable all interrupts
   IEC0bits.T1IE = 1;        // Enable interrupts for timer 1
   IEC0bits.IC1IE = 1;       // Enable interrupt
   IEC0bits.IC2IE = 1;
   IEC1bits.IC7IE = 1;
   IEC3bits.PWM1IE = 1;      // Enable PWM interrupts
   DISICNT = 0;
   Flags.MotorRunning = 1;   // Indicate that the motor is running
}
//--------------------------------------------------------------------------
void StopMotor(void)
{
  OVDCON = 0x0000;          // turn OFF every transistor
  // disable all interrupts
  IEC0bits.T1IE = 0;        // Disable interrupts for timer 1
  IEC0bits.IC1IE = 0;       // Disable interrupts on IC1
  IEC0bits.IC2IE = 0;       // Disable interrupts on IC2
  IEC1bits.IC7IE = 0;       // Disable interrupts on IC7
  IEC3bits.PWM1IE = 0;      // Disable PWM interrupts
  DISICNT = 0;
  Flags.MotorRunning = 0;// Indicate that the motor has been stopped
}
//---------------------------------------------------------------------
// Charge Bootstraps
//----------------------------------------------------------------------
void ChargeBootstraps(void)
{
  unsigned int i;
  OVDCON = 0x0015;       // Turn ON low side transistors to charge
  for (i = 0; i < 33330; i++);// 10 ms Delay at 20 MIPs
  PWMCON2bits.UDIS = 1;
  PDC1 = PTPER;        // Initialize as 0 voltage
  PDC2 = PTPER;        // Initialize as 0 voltage
  PDC3 = PTPER;        // Initialize as 0 voltage
  OVDCON = 0x3F00;     // Configure PWM0-5 to be governed by PWM module
  PWMCON2bits.UDIS = 0;
}
//-----------------------------------------------------------------------
void Delay(unsigned int delay_count)
{
  volatile int i;
  while (delay_count-- > 0)
  {
    for (i = 0;i < 1000;i++);
  }
}
//--------------------------------------------------------------------------
void SetPID(void)
{
 //------------------------------------------------
// Close loop PID parameters
//-----------------------------------------------
   Kp = Q15_Ftoi(0.1);   // P Gain
   Ki = Q15_Ftoi(0.01);  // I Gain
   Kd = Q15_Ftoi(0.001); // D Gain
}
//------------------------------------------------------------------------
void debug(void)
{
  Delay_ms(20);
  MotorSpeed = (signed int)(float_measuredspeed * 6000);
  if(Current_Direction == CCW){MotorSpeed = -MotorSpeed ;}
}
//---------------------------------------------------------------------------