/*************************************************************************
*  Space Vector Modulation dspIC33FJ128MC804
*  Port from MicroChip Application Note AN1017
*  February 2012.
*  PMSM Motor = Hurst 24V available @MCHP see Project Documentation
**************************************************************************/
#include "built_in.h"
#include "Config.h"
#include "ISR.h"
#include "IO_CAN.h"
//------------------------------------------------------------------------
extern struct MotorFlags
{
 unsigned MotorRunning :1;
 unsigned unused       :15;
}Flags;
//-----------------------------------------------------
//Module definitions
//------------------------------------------------------
#define S2 PORTAbits.RA8  // Button S2
#define S3 PORTBbits.RB4  // Button S3

//------------------------------------------------------
//Main Code
//------------------------------------------------------
void main()
{
  /***************************************************************************
 * Set PLL : Fosc = ((Fin/PLLPRE)*PLLDIV)/PLLPOST ;
 * Configure Oscillator to operate the device at 20Mhz
 * Fosc= Fin*M/(N1*N2), Fcy = Fosc/2
 * Fosc= 8*20/(2*2)= 40Mhz for 8M input clock
 * Fin = 8Mhz
 * Fcy = 20MIPS
******************************************************************************/
  PLLFBD =  18;           // M=20
  CLKDIVbits.PLLPOST = 0; // N1=2
  CLKDIVbits.PLLPRE = 0;  // N2=2
//*********************************************************
// IO Pin Mapping
//-----------------------------------------
   IOMap();
//----------------------------------------------
// Initialize Hall sensor inputs and Interrupts
//----------------------------------------------
   InitIC();
//----------------------------------------------
// Initialize ADC Module
//----------------------------------------------
   InitADC();
//----------------------------------------------
// Initialize MCPWM
//----------------------------------------------
   InitMCPWM();
//-----------------------------------------------
// Initialize Timer 1
//-----------------------------------------------
   InitTMR1();
//-----------------------------------------------
// Initialize Timer 3
//-----------------------------------------------
   InitTMR3(void);
//-----------------------------------------------
// Initialize CAN module
//-----------------------------------------------
   InitCAN();
//-----------------------------------------------
// Initialize PID controller
//------------------------------------------------
   SetPID();
//-----------------------------------------------
//  Continious main loop
//-----------------------------------------------
    Flags.MotorRunning = 0;
 while(1)
  {
    if ((!S2) && (!Flags.MotorRunning))
    {
      Delay(200);
      if(!S2)
       {
        RunMotor();  // Run motor if button is pressed and motor is stopped
        while(!S2);
       }
     }
   else if ((!S2) && (Flags.MotorRunning))
     {
       Delay(200);
       if(!S2)
       {
         StopMotor(); // Stop motor button pressed and motor is running
         while(!S2);
       }
     }
   /*--Motor Running send out Motor Data------*/
    if(Flags.MotorRunning){
        debug();
        WriteCAN();}
  }
}
//*********************************EOF**********************************